export type Edge<V> = [from: V, to: V];
/**
 * Port of Erlang's digraph
 * WIP
 *
 * @public
 * @category Structures
 */
export declare class DirectedGraph<V, L = unknown> {
    private cyclical;
    private vertices;
    private edges;
    constructor(options?: {
        cyclical?: boolean;
    });
    addVertex(vertex: V): DirectedGraph<V, L>;
    deleteVertex(vertex: V): boolean;
    deleteVertices(vertices: V[]): boolean[];
    private inEdgesIterator;
    private outEdgesIterator;
    private edgesIterator;
    /** return all edges fora given vertex in an unspecified order */
    getEdges(vertex: V): Edge<V>[];
    inDegree(vertex: V): number;
    outDegree(vertex: V): number;
    inEdges(vertex: V): Edge<V>[];
    outEdges(vertex: V): Edge<V>[];
    inNeighbors(vertex: V): V[];
    outNeighbors(vertex: V): V[];
    addEdge(from: V, to: V): this;
    deleteEdge(edge: Edge<V>): boolean;
    deleteEdges(edges: Edge<V>[]): boolean[];
    /** return all vertices */
    /** Returns number of vertices */
    numVertices(): number;
    /** Returns number of edges */
    numEdges(): number;
    /**
     * If a simple cycle of length two or more exists through a vertex, the cycle is returned as a list [V, ..., V] of vertices.
     * If a loop through the vertex exists, the loop is returned as a list [V]. If no cycles exist, undefined is returned.
     */
    getCycle(vertex: V): V[] | undefined;
    /**
     * Tries to find an as short as possible simple cycle through a vertex.
     * Returns the cycle as a list [V, ..., V] of vertices, or undefined if no simple cycle exists.
     * Notice that a loop through the vertex is returned as list [V, V].
     */
    getShortCycle(vertex: V): V[] | undefined;
    getPath(from: V, to: V): V[] | undefined;
    getShortPath(from: V, to: V): V[] | undefined;
    /**
     * Delete all paths found between two vertices.
     * Returns false if not paths found, otherwise returns true
     */
    deletePath(from: V, to: V): boolean;
    components(): V[][];
    /**
     * Returns true if and only if the DirectedGraph is acyclic.
     */
    isAcyclic(): boolean;
    /**
     * Returns true if and only if the DirectedGraph is a tree.
     */
    isTree(): boolean;
    reachable(vertices: V[]): V[];
    reachableNeighbors(vertices: V[]): V[];
    /**
     *
     */
    postOrder(): V[];
    /**
     *
     */
    preOrder(): V[];
    /**
     *
     */
    topsort(): V[];
    /**
     * TODO: find a better name
     * TODO: look into optimizing this for javascript
     */
    private onePath;
    private shortPath;
    private followPath;
    private postGenerate;
}
